from copy import copy
from typing import List, Union

from tablate.classes.bases.TablateApiItem import TablateApiItem
from tablate.library.calcs.gen_frame_name import gen_frame_name
from tablate.type.primitives import FrameDivider, ColumnDivider, OuterBorder, OuterPadding, OuterWidth, Background, \
    BackgroundPadding, HtmlPxMultiplier, Multiline, MaxLines, ColumnPadding, TextStyle, TextAlign, TextColor, FrameName
from tablate.type.type_input import GridColumnInput, HtmlOuterStylesInput, HtmlGridFrameStylesInput


class Grid(TablateApiItem):

    def __init__(self,
                 # TablateGrid args
                 columns: List[Union[str, GridColumnInput]],
                 name: FrameName = None,
                 frame_divider: FrameDivider = None,
                 background: Background = None,
                 background_padding: BackgroundPadding = None,
                 multiline: Multiline = None,
                 max_lines: MaxLines = None,
                 column_divider: ColumnDivider = None,
                 column_padding: ColumnPadding = None,
                 text_style: TextStyle = None,
                 text_align: TextAlign = None,
                 text_color: TextColor = None,
                 html_px_multiplier: HtmlPxMultiplier = None,
                 html_styles: HtmlGridFrameStylesInput = None,
                 # TablateApi args
                 outer_border: OuterBorder = None,
                 outer_padding: OuterPadding = None,
                 outer_width: OuterWidth = None,
                 html_outer_styles: HtmlOuterStylesInput = None) -> None:

        super().__init__(outer_border=outer_border,
                         outer_padding=outer_padding,
                         frame_divider=frame_divider,
                         outer_width=outer_width,
                         html_styles=html_outer_styles)

        name = gen_frame_name(name=name, type="grid", frame_dict=self._frame_list)

        args = copy(locals())
        del args["self"]

        self._frame_list[name] = {
            "name": name,
            "type": "grid",
            "options": args
        }



from typing import Optional

from tablate.library.formatters.html.style.attributes.space import space_attr
from tablate.type.primitives import HtmlSpacer, HtmlPxMultiplier


def padding_attr(column_padding: Optional[HtmlSpacer], html_px_multiplier: HtmlPxMultiplier):
    return space_attr(html_spacer=column_padding, html_px_multiplier=html_px_multiplier)[0]



from typing import Optional

from tablate.library.checkers.set_attr_resolver import set_attr_resolver
from tablate.type.defaults import text_style_default, text_align_default, text_color_default
from tablate.type.type_base import TextBase
from tablate.type.type_store import TextStore


def base_text_mapper(text_input: Optional[TextBase] = None,
                     text_defaults: Optional[TextBase] = None) -> TextStore:

    ####################################################################################################################

    text_input = text_input if text_input is not None else TextBase()
    text_defaults = text_defaults if text_defaults is not None else TextBase()

    ####################################################################################################################

    text_return = TextStore(text_style=set_attr_resolver(instance=text_input,
                                                         attr="text_style",
                                                         default=set_attr_resolver(instance=text_defaults,
                                                                                   attr="text_style",
                                                                                   default=text_style_default)),
                            text_align=set_attr_resolver(instance=text_input,
                                                         attr="text_align",
                                                         default=set_attr_resolver(instance=text_defaults,
                                                                                   attr="text_align",
                                                                                   default=text_align_default)),
                            text_color=set_attr_resolver(instance=text_input,
                                                         attr="text_color",
                                                         default=set_attr_resolver(instance=text_defaults,
                                                                                   attr="text_color",
                                                                                   default=text_color_default)))

    return text_return



from typing import Union, List, Optional, Tuple

from tablate.classes.options.html.style.utilities.base_values import styles_key
from tablate.classes.options.html.style.utilities.style_types import SelectorDictUnion, SelectorDictKeysUnion, StyleItem


def style_dict_key_builder(style_dict: dict, selector_dict: SelectorDictUnion, key: SelectorDictKeysUnion) -> Union[dict, list]:
    if key in selector_dict:
        if selector_dict[key] in style_dict:
            style_dict = style_dict[selector_dict[key]]
        else:
            style_dict[selector_dict[key]] = {}
            style_dict = style_dict[selector_dict[key]]
    return style_dict


def style_dict_css_append(style_dict: dict, css_item: StyleItem):
    if styles_key in style_dict:
        style_dict[styles_key].append(css_item)
    else:
        style_dict[styles_key] = [css_item]
    return style_dict



import tablate as tf
from tablate.tests.test_objs import column_list1, row_list1, column_list2a, row_list2, column_list2b, column_list2c, \
    column_list3, column_list4, column_list5, really_long_string

# new_frame = tf.Tablate(outer_border="thin", outer_padding=6)
new_frame = tf.Tablate()


new_frame.add_table_frame(column_list1, row_list1)

new_frame.add_table_frame(column_list1, row_list1, multiline=True)
new_frame.add_table_frame(column_list1, row_list1, multiline=True, max_lines=3)
new_frame.add_table_frame(column_list1, row_list1, multiline=True, max_lines=6)
new_frame.add_table_frame(column_list1, row_list1, multiline=True, max_lines=100)
new_frame.add_table_frame(column_list1, row_list1, max_lines=100)

new_frame.add_table_frame(column_list1, row_list1, multiline_header=True)
new_frame.add_table_frame(column_list1, row_list1, multiline_header=True, max_lines_header=2)
new_frame.add_table_frame(column_list1, row_list1, multiline_header=True, max_lines_header=100)
new_frame.add_table_frame(column_list1, row_list1, max_lines_header=100)


new_frame.add_text_frame("Default text")

new_frame.add_text_frame(really_long_string)
new_frame.add_text_frame(really_long_string, multiline=True)

new_frame.add_text_frame(really_long_string, multiline=False)
new_frame.add_text_frame(really_long_string, max_lines=3)

new_frame.add_text_frame(really_long_string, multiline=True, max_lines=3)
new_frame.add_text_frame(really_long_string, multiline=True, max_lines=100)

new_frame.add_text_frame(really_long_string, multiline=False, max_lines=3)
new_frame.add_text_frame(really_long_string, multiline=False, max_lines=100)

new_frame.add_text_frame("Example text frame with left indentation", multiline=False,text_align="left", max_lines=100)
new_frame.add_text_frame("Example text frame with center indentation", multiline=False,text_align="center")
new_frame.add_text_frame("Example text frame with right indentation", multiline=False,text_align="right")

new_frame.add_text_frame(666)




new_frame.add_text_frame(really_long_string)

new_frame.add_table_frame(column_list2a, row_list2, row_column_divider="blank", row_line_divider="none")

new_frame.add_table_frame(column_list2b, row_list2, row_column_divider="double", row_line_divider="double", frame_divider="blank")

new_frame.add_table_frame(column_list2c, row_list2, row_column_divider="blank", row_line_divider="blank", header_frame_divider="thin", header_column_divider="double", frame_divider="thin")

new_frame.add_table_frame(column_list2c, row_list2, row_column_divider="blank", row_line_divider="none", header_frame_divider="none", header_column_divider="double", frame_divider="double")
new_frame.add_table_frame(column_list2c, row_list2, row_column_divider="thin", row_line_divider="thin", header_frame_divider="thin", header_column_divider="double", frame_divider="thick")
new_frame.add_table_frame(column_list2c, row_list2, row_column_divider="thick", row_line_divider="thick", header_frame_divider="thick", header_column_divider="double", frame_divider="thin")

new_frame.add_table_frame(column_list2c, row_list2, row_column_divider="double", row_line_divider="double", header_frame_divider="double", header_column_divider="double", frame_divider="none")
new_frame.add_table_frame(column_list2c, row_list2)

new_frame.add_grid_frame(column_list3, max_lines=1)
new_frame.add_grid_frame(column_list4, max_lines=6)
new_frame.add_grid_frame(column_list4, max_lines=100)
new_frame.add_grid_frame(column_list5)

some_set = {1,2,3}
some_set.add(1)

some_set.add(4)



from tablate.library.ascii.chars.line_h import h_line
from tablate.library.ascii.chars.line_v import v_line
from tablate.library.ascii.chars.matrix_cross import cross_matrix
from tablate.library.checkers.set_key_resolver import set_key_resolver
from tablate.library.formatters.console.ascii_styler import ascii_text_styler, ascii_terminator
from tablate.library.formatters.console.concat_string import concat_string
from tablate.library.initializers.append.append import process_frame_list
from tablate.library.initializers.globals_init import globals_init
from tablate.type.type_global import Globals
from tablate.type.type_input import FrameDictList


def list_frames(frame_list: FrameDictList, globals_dict: dict):
    print()
    # top_index = f"{' '}{h_line['thin'] * ((len(frame_list) // 10) + 3)}{cross_matrix['blank']['thin']['thin']}"
    # top_name = f"{h_line['thin'] * 22}{cross_matrix['blank']['thin']['thin']}"
    # top_type = f"{h_line['thin'] * 8}{cross_matrix['blank']['thin']['thin']}"
    # top_cols = f"{h_line['thin'] * 7}{cross_matrix['blank']['thin']['thin']}"
    # top_rows = f"{h_line['thin'] * 7}{cross_matrix['blank']['thin']['thin']}"
    # top_globals = f"{h_line['thin'] * 42}"
    # print(f"{top_index}{top_name}{top_type}{top_cols}{top_rows}{top_globals}")
    divider = v_line["thin"]
    padding = " "
    header_string = f"{padding} {' ' * ((len(frame_list) // 10) + 1)}{padding}"
    frame_name, name_ws = concat_string(string="Name",
                                        width=20,
                                        padding=0,
                                        trunc_value="")
    frame_name = ascii_text_styler(string=frame_name, column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += f"{divider}{padding}{frame_name}{' ' * name_ws}{padding}"
    frame_type, type_ws = concat_string(string="Type",
                                        width=6,
                                        padding=0,
                                        trunc_value="")
    frame_type = ascii_text_styler(string=frame_type, column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += f"{divider}{padding}{frame_type}{' ' * type_ws}{padding}"
    frame_cols, cols_ws = concat_string(string="Cols.",
                                        width=5,
                                        padding=0,
                                        trunc_value="")
    frame_cols = ascii_text_styler(string=frame_cols, column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += f"{divider}{padding}{frame_cols}{' ' * cols_ws}{padding}"
    frame_rows, rows_ws = concat_string(string="Rows",
                                        width=5,
                                        padding=0,
                                        trunc_value="")
    frame_rows = ascii_text_styler(string=frame_rows, column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += f"{divider}{padding}{' ' * rows_ws}{frame_rows}{padding}"
    frame_options = ascii_text_styler(string="Options", column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += f"{divider}{padding}{frame_options}{padding}"
    default_options = ascii_text_styler(string="[defaults]", column_dict={"text_style": "bold"}) + ascii_terminator()
    header_string += default_options
    print(header_string)
    divider_index = f"{padding}{h_line['thin'] * ((len(frame_list) // 10) + 3)}{cross_matrix['thin']['thin']['thin']}"
    divider_name = f"{h_line['thin'] * 22}{cross_matrix['thin']['thin']['thin']}"
    divider_type = f"{h_line['thin'] * 8}{cross_matrix['thin']['thin']['thin']}"
    divider_cols = f"{h_line['thin'] * 7}{cross_matrix['thin']['thin']['thin']}"
    divider_rows = f"{h_line['thin'] * 7}{cross_matrix['thin']['thin']['thin']}"
    divider_globals = f"{h_line['thin'] * 42}"
    print(f"{divider_index}{divider_name}{divider_type}{divider_cols}{divider_rows}{divider_globals}")
    ####################################################################################################################
    global_options = globals_init(**globals_dict)
    processed_frame_list = process_frame_list(frame_list=frame_list, global_options=global_options)
    processed_frame_index = 0
    ####################################################################################################################

    for frame_index, (frame_key, frame_item) in enumerate(frame_list.items()):
        item_string = f"{padding}"
        index, index_ws = concat_string(string=frame_index,
                                        width=(len(frame_list) // 10) + 1,
                                        padding=0,
                                        trunc_value="")
        item_string += f"{padding}{' ' * index_ws}{index}{padding}"
        item_name, name_ws = concat_string(string=frame_item["name"],
                                           width=20,
                                           padding=0,
                                           trunc_value="...")
        item_string += f"{divider}{padding}{item_name}{' ' * name_ws}{padding}"
        item_type = frame_item["type"].capitalize() if frame_item["type"] != "table_body" else "Table"
        item_type, type_ws = concat_string(string=item_type,
                                           width=6,
                                           padding=0,
                                           trunc_value="")
        item_string += f"{divider}{padding}{item_type}{' ' * type_ws}{padding}"
        item_cols, cols_ws = concat_string(string=len(frame_item["options"]["columns"]),
                                           width=5,
                                           padding=0,
                                           trunc_value="")
        item_cols = item_cols if frame_item["type"] != "text" else "-"
        item_string += f"{divider}{padding}{' ' * cols_ws}{item_cols}{padding}"
        item_rows_list = set_key_resolver(instance=frame_item["options"], key="row_list", default=[])
        item_rows, rows_ws = concat_string(string=len(item_rows_list),
                                           width=5,
                                           padding=0,
                                           trunc_value="")
        item_rows = item_rows if frame_item["type"] == "table" else "-"
        item_string += f"{divider}{padding}{' ' * rows_ws}{item_rows}{padding}"
        item_options_list = []
        default_options_list = []
        item_options_list.append("hide_header") if "hide_header" in frame_item["options"] and frame_item["options"]["hide_header"] is True else None
        default_options_list.append("hide_header") if hasattr(processed_frame_list[processed_frame_index].frame_styles, "hide_header") and processed_frame_list[processed_frame_index].frame_styles.hide_header is True else None
        item_options_list.append("multiline") if frame_item["options"]["multiline"] is True else None
        default_options_list.append("multiline") if processed_frame_list[processed_frame_index].frame_styles.multiline is True else None
        item_options_list.append(f"max_lines: {frame_item['options']['max_lines']}") if frame_item["options"]["max_lines"] is not None else None
        default_options_list.append("max_lines") if processed_frame_list[processed_frame_index].frame_styles.max_lines is not None else None
        duplicate_options_list = []
        for default_option_index, default_option_item in enumerate(default_options_list):
            if default_option_item in item_options_list:
                duplicate_options_list.append(default_option_index)
        duplicate_options_list.reverse()
        for duplicate_option in duplicate_options_list:
            default_options_list.pop(duplicate_option)
        item_options_string = f"{', '.join(item_options_list)}{padding}" if len(item_options_list) > 0 else ""
        duplicate_option_string = f"[{', '.join(default_options_list)}]" if len(default_options_list) > 0 else ""
        item_string += f"{divider}{padding}{item_options_string}{duplicate_option_string}"
        print(item_string)
        processed_frame_index += 1 if processed_frame_list[processed_frame_index].type != "table_header" else 2
    print()

    # todo: please fix this!!!


def background_color_attr(color: str):
    colors_map = {
        "white": "white",
        "grey": "lightgray",
        "red": "red",
        "green": "limegreen",
        "yellow": "yellow",
        "blue": "skyblue",
        "magenta": "violet",
        "cyan": "cyan",
        "dark_red": "darkred",
        "dark_green": "green",
        "dark_yellow": "khaki",
        "dark_blue": "royalblue",
        "dark_magenta": "mediumorchid",
        "dark_cyan": "darkturquoise",
        "dark_grey": "darkgray",
        "black": "black",
    }
    return colors_map[color]


def text_color_attr(color):
    colors_map = {
        "white": "white",

        "grey": "darkgray",
        "red": "red",
        "green": "green",
        "yellow": "gold",
        "blue": "dodgerblue",
        "magenta": "orchid",
        "cyan": "turquoise",

        "dark_red": "darkred",
        "dark_green": "darkgreen",
        "dark_yellow": "orange",
        "dark_blue": "darkblue",
        "dark_magenta": "darkviolet",
        "dark_cyan": "darkcyan",
        "dark_grey": "dimgray",

        "black": "black",

    }
    return colors_map[color]



from tablate.library.checkers.is_last_element import is_last_element
from tablate.library.formatters.console.row_line_divider import row_line_divider
from tablate.library.formatters.console.row_outer_border import row_outer_border
from tablate.library.renderers.console.frames.render_console_frame_grid import render_console_single_line_grid, \
    render_console_multi_line_grid
from tablate.library.renderers.console.frames.render_console_frame_table import render_console_table_frame
from tablate.type.type_store import FrameStoreList
from tablate.type.type_global import Globals


def render_console_frames(frame_list: FrameStoreList, global_options: Globals) -> str:

    return_string = ""

    for frame_index, frame_item in enumerate(frame_list):

        if frame_item.type == "grid" or frame_item.type == "table_header" or frame_item.type == "text":
            if frame_item.frame_styles.multiline is False or frame_item.frame_styles.max_lines == 1:
                return_string += render_console_single_line_grid(grid_frame_dict=frame_item, global_options=global_options)
            else:
                return_string += render_console_multi_line_grid(grid_frame_dict=frame_item, global_options=global_options)

        if frame_item.type == "table_body":
            return_string += render_console_table_frame(table_frame_store=frame_item, global_options=global_options)

        if not is_last_element(frame_index, frame_list):
            if frame_list[frame_index].frame_styles.frame_divider != 'none':
                frame_divider_inner = row_line_divider(column_list_top=frame_list[frame_index].column_list,
                                                       column_list_bottom=frame_list[frame_index + 1].column_list,
                                                       divider=frame_list[frame_index].frame_styles.frame_divider)
                return_string += row_outer_border(row_string=frame_divider_inner,
                                                  global_options=global_options,
                                                  row_divider=frame_item.frame_styles.frame_divider)

    return return_string



from typing import Optional

from tablate.library.checkers.set_attr_resolver import set_attr_resolver
from tablate.type.defaults import html_outer_width_default, html_px_multiplier_default, outer_border_default, \
    outer_padding_default, outer_width_default, html_divider_weight_default, html_padding_default
from tablate.type.primitives import HtmlPxMultiplier
from tablate.type.type_base import HtmlOuterBase, OuterBase
from tablate.type.type_store import HtmlOuterStore


def html_outer_mapper(html_outer_input: Optional[HtmlOuterBase] = None,
                      html_outer_defaults: Optional[HtmlOuterBase] = None,
                      base_outer_defaults: OuterBase = None,
                      html_px_multiplier: HtmlPxMultiplier = html_px_multiplier_default) -> HtmlOuterStore:
    if html_outer_input is None:
        html_outer_input = HtmlOuterBase()

    # html_outer_padding_default = outer_defaults.outer_padding * html_px_multiplier_default

    html_outer_border_weight = set_attr_resolver(
        instance=html_outer_input,
        attr="html_outer_border_weight",
        default=set_attr_resolver(
            instance=html_outer_defaults,
            attr="html_outer_border_weight",
            default=html_divider_weight_default))
    html_outer_border_style = set_attr_resolver(
        instance=html_outer_input,
        attr="html_outer_border_style",
        default=set_attr_resolver(
            instance=html_outer_defaults,
            attr="html_outer_border_style",
            default=set_attr_resolver(
                instance=base_outer_defaults,
                attr="outer_border",
                default=outer_border_default)))
    html_outer_padding = set_attr_resolver(
        instance=html_outer_input,
        attr="html_outer_padding",
        default=set_attr_resolver(
            instance=html_outer_defaults,
            attr="html_outer_padding",
            default=set_attr_resolver(
                instance=base_outer_defaults,
                attr="outer_padding",
                default=outer_padding_default)*html_padding_default))
    html_outer_width = set_attr_resolver(
        instance=html_outer_input,
        attr="html_outer_width",
        default=set_attr_resolver(
            instance=html_outer_defaults,
            attr="html_outer_width",
            default=html_outer_width_default))

    html_outer_return = HtmlOuterStore(html_outer_border_weight=html_outer_border_weight,
                                       html_outer_border_style=html_outer_border_style,
                                       html_outer_padding=html_outer_padding,
                                       html_outer_width=html_outer_width,
                                       html_px_multiplier=html_px_multiplier)

    return html_outer_return



from typing import Callable, Union, List, Tuple, Optional

from tablate.classes.options.html.style.mixins.AddStyleMixin import AddStyleMixin
from tablate.classes.options.html.style.mixins.ClassNameMixin import ClassNameMixin
from tablate.classes.options.html.style.utilities.style_types import SelectorDictUnion, StyleItem
from tablate.type.primitives import TextAlign


class TextStyler(ClassNameMixin, AddStyleMixin):

    def __init__(self, selector: SelectorDictUnion,
                 create_style: Callable[[SelectorDictUnion, StyleItem], None]) -> None:
        self._selector_dict: SelectorDictUnion = selector
        self._create_style = create_style

    def alignment(self, align=TextAlign):
        pass



from tablate.classes.options.html.style.subclasses.ElementStyler import ElementStyler


def html_row_head_formatter(row_styler: ElementStyler) -> str:
    return f'<tr class="{row_styler.generate_class_names()}">'


def html_row_foot_formatter() -> str:
    return '</tr>'



from dataclasses import dataclass, field
from typing import Literal, Union, List, Dict

from tablate.type.defaults import background_padding_default, html_px_multiplier_default, frame_name_default
from tablate.type.primitives import TextString, TableRowKey, HtmlPxMultiplier, BackgroundPadding, FrameName, HideHeader
from tablate.type.type_base import ColumnBase, TextBase, HtmlTextBase, FrameBase, HtmlColumnBase, \
    HtmlTableRowsBase, TableRowsBase, HtmlFrameBase, HtmlOuterBase, OuterBase
from tablate.type.type_input import GridColumnInput, TableColumnInput, BaseColumnInput

########################################################################################################################
# FrameDicts ###########################################################################################################
########################################################################################################################


@dataclass
class OuterStore(OuterBase):
    background_padding: BackgroundPadding = background_padding_default


@dataclass
class HtmlOuterStore(HtmlOuterBase):
    html_px_multiplier: HtmlPxMultiplier = html_px_multiplier_default


@dataclass
class HtmlFrameStore(HtmlFrameBase):
    html_px_multiplier: HtmlPxMultiplier = html_px_multiplier_default


FrameStore = FrameBase
ColumnStore = ColumnBase
TextStore = TextBase

HtmlColumnStore = HtmlColumnBase
HtmlTextStore = HtmlTextBase

TableRowsStore = TableRowsBase
HtmlTableRowsStore = HtmlTableRowsBase


class BaseColumnStore(BaseColumnInput):
    background_padding: BackgroundPadding = background_padding_default


@dataclass
class BaseFrameStore:
    frame_styles: FrameStore = field(default_factory=FrameStore)
    column_styles: ColumnStore = field(default_factory=ColumnStore)
    text_styles: TextStore = field(default_factory=TextStore)
    html_frame_styles: HtmlFrameStore = field(default_factory=HtmlFrameStore)
    html_column_styles: HtmlColumnStore = field(default_factory=HtmlColumnStore)
    html_text_styles: HtmlTextStore = field(default_factory=HtmlTextStore)


# Grid FrameDict #######################################################################################################


GridColumnStore = BaseColumnStore


@dataclass()
class GridFrameStore(BaseFrameStore):
    type: Union[Literal["grid"], Literal["text"]] = "text"
    name: FrameName = frame_name_default
    column_list: List[GridColumnStore] = field(default_factory=list)


# Table FrameDict ######################################################################################################

@dataclass()
class TableHeaderFrameStore(BaseFrameStore):
    type: Literal["table_header"] = "table_header"
    name: FrameName = frame_name_default
    column_list: List[BaseColumnStore] = field(default_factory=list)


@dataclass()
class TableBodyFrameStore(BaseFrameStore):
    type: Literal["table_body"] = "table_body"
    name: FrameName = frame_name_default
    hide_header: HideHeader = False
    column_list: List[TableColumnInput] = field(default_factory=list)
    row_list: List[Dict[TableRowKey, TextString]] = field(default_factory=list)
    row_styles: TableRowsStore = field(default_factory=TableRowsStore)
    html_row_styles: HtmlTableRowsStore = field(default_factory=HtmlTableRowsStore)


# FrameDict List #######################################################################################################

FrameStoreUnion = Union[GridFrameStore, TableHeaderFrameStore, TableBodyFrameStore]
FrameStoreList = List[FrameStoreUnion]



from dataclasses import dataclass, field
from typing import TypedDict, Dict, List, NotRequired, Optional, Literal

from tablate.type.primitives import TextString, FrameDivider, ColumnWidth, TextStyle, TextAlign, Background, \
    ColumnPadding, TextColor, ColumnDivider, HtmlDividerWeight, HtmlColumnDividerStyle, HtmlColumnPadding, \
    HtmlTextStyle, HtmlTextAlign, HtmlTextColor, HtmlBackground, FrameName
from tablate.type.type_base import ColumnBase, TextBase, HtmlTextBase, TableRowsBase, \
    HtmlColumnBase, HtmlTableRowsBase, HtmlOuterBase, HtmlFrameBase, FrameBase

HtmlOuterStylesInput = HtmlOuterBase

FrameStylesInput = FrameBase

HtmlFrameStylesInput = HtmlFrameBase


class HtmlColumnInput(TypedDict):
    padding: NotRequired[HtmlColumnPadding]
    divider_style: NotRequired[HtmlColumnDividerStyle]
    divider_weight: NotRequired[HtmlDividerWeight]
    text_style: NotRequired[HtmlTextStyle]
    text_align: NotRequired[HtmlTextAlign]
    text_color: NotRequired[HtmlTextColor]
    background: NotRequired[HtmlBackground]


class BaseColumnStyles(TypedDict):
    width: NotRequired[ColumnWidth]
    padding: NotRequired[ColumnPadding]
    divider: NotRequired[ColumnDivider]
    text_style: NotRequired[TextStyle]
    text_align: NotRequired[TextAlign]
    text_color: NotRequired[TextColor]
    background: NotRequired[Background]
    html_styles: NotRequired[Optional[HtmlColumnInput]]


class BaseColumnInput(BaseColumnStyles):
    string: TextString


########################################################################################################################

GridColumnInput = BaseColumnInput


class TableColumnInput(BaseColumnInput):
    key: str


TableRowsDataInputDict = Dict[str, TextString]


########################################################################################################################

ColumnStylesInput = ColumnBase
TextStylesInput = TextBase

HtmlColumnStylesInput = HtmlColumnBase
HtmlTextStylesInput = HtmlTextBase

########################################################################################################################
########################################################################################################################
########################################################################################################################


@dataclass
class BaseStylesInput:
    frame_styles: Optional[FrameBase] = None
    column_styles: Optional[ColumnBase] = None
    text_styles: Optional[TextBase] = None


TableHeaderFrameStylesInput = BaseStylesInput


@dataclass
class TableBodyFrameStylesInput(BaseStylesInput):
    row_styles: Optional[TableRowsBase] = None


########################################################################################################################
########################################################################################################################
########################################################################################################################

@dataclass
class HtmlStylesInput:
    html_frame_styles: Optional[HtmlFrameBase] = None
    html_column_styles: Optional[HtmlColumnBase] = None
    html_text_styles: Optional[HtmlTextBase] = None


########################################################################################################################

@dataclass
class HtmlTextFrameStylesInput:
    html_frame_styles: Optional[HtmlFrameBase] = None
    html_text_styles: Optional[HtmlTextBase] = None


HtmlGridFrameStylesInput = HtmlStylesInput


@dataclass
class HtmlTableFrameStylesInput(HtmlStylesInput):
    html_row_styles: Optional[HtmlTableRowsBase] = None


HtmlTableBodyFrameStylesInput = HtmlTableFrameStylesInput
HtmlTableHeaderFrameStylesInput = HtmlStylesInput

########################################################################################################################
########################################################################################################################
########################################################################################################################

class FrameDict(TypedDict):
    name: FrameName
    type: Literal["text", "grid", "table"]
    options: dict


FrameDictList = Dict[str, FrameDict]


from copy import copy, deepcopy
from typing import Callable, Tuple

from IPython.core.display import HTML
from IPython.core.display_functions import display

from tablate.classes.bases.TablateBase import TablateBase
from tablate.library.renderers.console.render_console import render_console
from tablate.library.renderers.html.render_html import render_html

from tablate.type.primitives import OuterBorder, FrameDivider, OuterWidth, OuterPadding, Background, HtmlPxMultiplier, \
    BackgroundPadding
from tablate.type.type_input import HtmlOuterStylesInput, ColumnStylesInput, TextStylesInput, \
    HtmlFrameStylesInput, HtmlColumnStylesInput, HtmlTextStylesInput


class TablateApiBase(TablateBase):

    def __init__(self,
                 outer_border: OuterBorder = None,
                 outer_padding: OuterPadding = None,
                 outer_width: OuterWidth = None,

                 frame_divider: FrameDivider = None,
                 background: Background = None,
                 background_padding: BackgroundPadding = None,

                 html_px_multiplier: HtmlPxMultiplier = None,
                 html_styles: HtmlOuterStylesInput = None,

                 column_styles: ColumnStylesInput = None,
                 text_styles: TextStylesInput = None,

                 html_frame_styles: HtmlFrameStylesInput = None,

                 html_column_styles: HtmlColumnStylesInput = None,
                 html_text_styles: HtmlTextStylesInput = None) -> None:

        args = copy(locals())
        del args["self"]

        self._globals_dict = args
        self._css_injection = []
        self._frame_list = {}

    # def __is_ipython(self):
    #     return hasattr(__builtins__, "__IPYTHON__")

    # def list_frames(self):
    #     print_frame_list(self._frame_list)

    def apply(self, function: Callable[[dict, dict],Tuple[dict, dict]]):
        frame_list_copy = deepcopy(self._frame_list)
        global_options_copy = deepcopy((self._globals_dict))
        try:
            frame_dict, global_dict = function(frame_list_copy, global_options_copy)
            if frame_dict is not None:
                self._frame_list = frame_dict
            if global_options_copy is not None:
                self._globals_dict = global_dict
        except:
            self._frame_list = frame_list_copy
            self._globals_dict = global_options_copy

    def apply_style(self, selector: str, css: str, sub_selector: str = None):
        self._css_injection.append({"selector": selector, "css": css, "sub_selector": sub_selector})

    def to_string(self) -> str:
        return render_console(frame_list=self._frame_list, global_options=self._globals_dict)

    def print(self) -> None:
        print(self.to_string())

    def __repr__(self) -> str:
        return self.to_string()

    def to_html(self) -> str:
        return render_html(frame_list=self._frame_list, globals_dict=self._globals_dict, css_injection=self._css_injection)

    def _repr_html_(self) -> str:
        return display(HTML(self.to_html()))



from tablate.library.ascii.chars.corners import top_left, top_right
from tablate.library.formatters.console.row_line_divider import row_line_divider
from tablate.type.type_store import FrameStoreList
from tablate.type.type_global import Globals


def render_console_head(frame_list: FrameStoreList, global_options: Globals) -> str:

    return_string = ""

    frame_border = global_options.console.outer_styles.outer_border
    outer_padding = global_options.console.outer_styles.outer_padding

    top_border_inner = row_line_divider(column_list_top=[],
                                        column_list_bottom=frame_list[0].column_list,
                                        divider=frame_border)


    return_string += f"{' ' * outer_padding}{top_left[frame_border]}{top_border_inner}{top_right[frame_border]}\n"

    return return_string


from typing import Union, Callable, Tuple

from tablate.classes.bases.TablateApiBase import TablateApiBase


class TablateApiItem(TablateApiBase):

    def name(self):
        for frame_key, frame_item in self._frame_list.items():
            print(frame_key)

    def rename(self, new_name: str):
        if new_name is not None:
            for frame_key, frame_item in self._frame_list.items():
                frame_item["name"] = new_name
                self._frame_list[new_name] = frame_item
                del self._frame_list[frame_key]
                break


    def to_dict(self):
        for frame_key, frame_item in self._frame_list.items():
            if frame_item["type"] == "text":
                return {
                    "text": [frame_item["options"]["text"]]
                }
            if frame_item["type"] == "grid":
                return_dict = {}
                for column_index, column_item in enumerate(frame_item["options"]["columns"]):
                    return_dict[column_index] = [column_item["string"]]
                return return_dict
            if frame_item["type"] == "table":
                return_dict = {}
                for column_item in frame_item["options"]["columns"]:
                    return_dict[column_item["key"]] = []
                    for row_item in frame_item["options"]["rows"]:
                        return_dict[column_item["key"]].append(row_item[column_item["key"]])
                return return_dict



from tablate.library.initializers.grid_init import grid_init
from tablate.library.initializers.table_init import table_init
from tablate.library.initializers.text_init import text_init
from tablate.type.primitives import FrameNameList
from tablate.type.type_global import Globals
from tablate.type.type_store import FrameStoreUnion, FrameStoreList


def process_frame_list(frame_list: dict, global_options: Globals) -> FrameStoreList:
    processed_frame_list: FrameStoreList = []
    for frame_key, frame_item in frame_list.items():
        if frame_item["type"] == "text":
            processed_frame_list.append(text_init(**frame_item["options"], global_optiona=global_options))
        if frame_item["type"] == "grid":
            processed_frame_list.append(grid_init(**frame_item["options"], global_options=global_options))
        if frame_item["type"] == "table":
            table_header, table_body = table_init(**frame_item["options"], global_options=global_options)
            if table_header:
                processed_frame_list.append(table_header)
            processed_frame_list.append(table_body)
    return processed_frame_list



from copy import deepcopy, copy
from typing import Union, Callable, Tuple

from tablate.api.modules.Table import Table
from tablate.classes.bases.TablateApiBase import TablateApiBase
from tablate.classes.classes import TablateUnion
from tablate.classes.helpers.get_frame import get_frame
from tablate.classes.helpers.list_frame import list_frames
from tablate.library.calcs.gen_frame_name import gen_frame_name
from tablate.type.primitives import FrameName, FrameDivider, Multiline, MaxLines, Background, BackgroundPadding, \
    HideHeader, ColumnDivider, ColumnPadding, TextStyle, TextAlign, TextColor, HtmlPxMultiplier, OuterBorder, \
    OuterPadding, OuterWidth
from tablate.type.type_input import TableHeaderFrameStylesInput, TableBodyFrameStylesInput, HtmlTableFrameStylesInput, \
    HtmlTableHeaderFrameStylesInput, HtmlTableBodyFrameStylesInput, HtmlOuterStylesInput


class TablateApiSet(TablateApiBase):

    def list_frames(self):
        list_frames(self._frame_list, self._globals_dict)

    def get_frame(self, selector: Union[int, str], apply_globals: bool = False):
        if apply_globals:
            return deepcopy(get_frame(frame_list=self._frame_list, selector=selector, global_options=self._globals_dict))
        else:
            return deepcopy(get_frame(frame_list=self._frame_list, selector=selector))

    def from_dict(self,
                  dict_object: dict,
                  name: FrameName = None,
                  capitalize_keys: bool = True,
                  frame_divider: FrameDivider = None,
                  multiline: Multiline = None,
                  max_lines: MaxLines = None,
                  background: Background = None,
                  background_padding: BackgroundPadding = None,

                  multiline_header: Multiline = None,
                  max_lines_header: MaxLines = None,
                  hide_header: HideHeader = None,

                  column_divider: ColumnDivider = None,
                  column_padding: ColumnPadding = None,
                  header_base_divider: FrameDivider = None,

                  row_line_divider: FrameDivider = None,
                  odd_row_background: Background = None,
                  even_row_background: Background = None,

                  text_style: TextStyle = None,
                  text_align: TextAlign = None,
                  text_color: TextColor = None,

                  header_styles: TableHeaderFrameStylesInput = None,
                  body_styles: TableBodyFrameStylesInput = None,

                  html_px_multiplier: HtmlPxMultiplier = None,
                  html_styles: HtmlTableFrameStylesInput = None,

                  html_header_styles: HtmlTableHeaderFrameStylesInput = None,
                  html_body_styles: HtmlTableBodyFrameStylesInput = None):

        name = gen_frame_name(name=name, type="table", frame_dict=self._frame_list)

        args = deepcopy(locals())
        del args["self"]
        del args["dict_object"]
        del args["capitalize_keys"]

        columns = []
        rows = []
        for col_index, (col_key, col_value) in enumerate(dict_object.items()):
            for row_index, (value_index, value) in enumerate(col_value.items()):
                if col_index == 0:
                    rows.append({})
                    if type(value_index) == tuple:
                        for i in range(0, len(value_index)):
                            if row_index == 0:
                                columns.append({"key": f"i{i}", "width": "10%", "text_align": "right"})
                            rows[row_index][f"i{i}"] = value_index[i]
                    else:
                        if row_index == 0:
                            columns.append({"key": "i", "width": "10%", "text_align": "right"})
                        rows[row_index]["i"] = value_index
                rows[row_index][col_key.title() if capitalize_keys else col_key] = value
            columns.append({"key": col_key.title() if capitalize_keys else col_key})
        new_table = Table(columns=columns, rows=rows, **args)
        # todo: fix column widths... ensure no bugs if millions of index columns
        # todo: for later allow specific column widths
        self._frame_list[name] = {"name": name, "type": "table", "options": {"columns":columns, "rows":rows, **args}}




    def remove_frame(self, selector: Union[int, str]):
        for frame_index, (frame_key, frame_item) in enumerate(self._frame_list.items()):
            if (type(selector) == int and selector == frame_index) or (type(selector) == str and selector == frame_key):
                del self._frame_list[frame_key]
                break

    def replace_frame(self, selector: Union[int, str], new_frame: TablateUnion, new_name: str = None):
        new_frame = deepcopy(new_frame)
        for frame_index, (frame_key, frame_item) in enumerate(self._frame_list.items()):
            if (type(selector) == int and selector == frame_index) or (type(selector) == str and selector == frame_key):
                for new_frame_key, new_frame_item in new_frame._frame_list.items():
                    new_name = new_name if new_name is not None else new_frame_key
                    new_name = gen_frame_name(name=new_name, type=new_frame_item["type"], frame_dict=self._frame_list, ensure_unique=True)
                    new_frame_item["name"] = new_name
                    new_frame_item["options"]["name"] = new_name
                    self._frame_list = {key if key != frame_key else new_name: value for key, value in self._frame_list.items()}
                    self._frame_list[new_name] = new_frame_item
                    break



from typing import Dict

from tablate.type.primitives import Colors

ascii_background_colors: Dict[Colors, str] = {
    "default": "",
    "white": "107",
    "grey": "47",
    "red": "101",
    "green": "102",
    "yellow": "103",
    "blue": "104",
    "magenta": "105",
    "cyan": "106",
    "dark_red": "41",
    "dark_green": "42",
    "dark_yellow": "43",
    "dark_blue": "44",
    "dark_magenta": "45",
    "dark_cyan": "46",
    "dark_grey": "100",
    "black": "40",
}


from tablate.library.initializers.append.append import process_frame_list
from tablate.library.initializers.globals_init import globals_init
from tablate.library.initializers.text_init import text_init
from tablate.library.initializers.grid_init import grid_init
from tablate.library.initializers.table_init import table_init
from tablate.library.renderers.console.render_console_foot import render_console_foot
from tablate.library.renderers.console.render_console_frames import render_console_frames
from tablate.library.renderers.console.render_console_head import render_console_head
from tablate.type.type_store import FrameStoreList
from tablate.type.type_global import Globals


def render_console(frame_list: dict, global_options: dict) -> str:
    global_options = globals_init(**global_options)
    processed_frame_list = process_frame_list(frame_list=frame_list, global_options=global_options)
    return_string = ""
    if len(processed_frame_list) > 0:
        return_string += render_console_head(frame_list=processed_frame_list, global_options=global_options)
        return_string += render_console_frames(frame_list=processed_frame_list, global_options=global_options)
        return_string += render_console_foot(frame_list=processed_frame_list, global_options=global_options)
    return return_string



from typing import Optional

from tablate.library.checkers.set_attr_resolver import set_attr_resolver
from tablate.type.defaults import column_divider_default, background_default, column_padding_default, \
    html_px_multiplier_default, html_padding_default, html_divider_weight_default
from tablate.type.primitives import HtmlPxMultiplier
from tablate.type.type_base import HtmlColumnBase, ColumnBase
from tablate.type.type_store import HtmlColumnStore


def html_column_mapper(html_columns_input: Optional[HtmlColumnBase] = None,
                       html_column_defaults: Optional[HtmlColumnBase] = None,
                       base_column_defaults: Optional[ColumnBase] = None) -> HtmlColumnStore:
    if html_columns_input is None:
        html_columns_input = HtmlColumnBase()

    html_divider_weight = set_attr_resolver(
        instance=html_columns_input,
        attr="html_divider_weight",
        default=set_attr_resolver(
            instance=html_column_defaults,
            attr="html_divider_weight",
            default=html_divider_weight_default))
    html_divider_style = set_attr_resolver(
        instance=html_columns_input,
        attr="html_divider_style",
        default=set_attr_resolver(
            instance=html_column_defaults,
            attr="html_divider_style",
            default=set_attr_resolver(
                instance=base_column_defaults,
                attr="divider",
                default=column_divider_default)))
    html_padding = set_attr_resolver(
        instance=html_columns_input,
        attr="html_padding",
        default=set_attr_resolver(
            instance=html_column_defaults,
            attr="html_padding",
            default=(set_attr_resolver(
                instance=base_column_defaults,
                attr="padding",
                default=html_padding_default) * html_padding_default)))

    columns_return = HtmlColumnStore(html_divider_weight=html_divider_weight,
                                     html_divider_style=html_divider_style,
                                     html_padding=html_padding)

    return columns_return

# todo: figure out a better way of doing this. (at the moment the inner checks are being performed before the base checks



from dataclasses import dataclass, field
from typing import Optional

from tablate.type.defaults import frame_divider_default, max_lines_default, multiline_default, background_default, \
    trunc_value_default, column_divider_default, column_padding_default, text_style_default, text_align_default, \
    text_color_default, row_line_divider_default, html_divider_weight_default, outer_border_default, \
    outer_padding_default, outer_width_default, html_outer_width_default, html_text_size_default, \
    html_background_default, html_max_lines_default, html_multiline_default, html_frame_divider_default, \
    html_outer_border_default, html_outer_padding_default, html_text_style_default, html_text_color_default, \
    html_text_align_default, html_row_line_divider_style_default
from tablate.type.primitives import FrameDivider, OuterBorder, TextStyle, TextAlign, ColumnWidth, \
    ColumnPadding, MaxLines, Multiline, OuterWidth, OuterPadding, HtmlTextStyle, HtmlTextColor, HtmlTextSize, \
    HtmlColumnDividerStyle, HtmlColumnPadding, HtmlBackground, HtmlFrameDivider, \
    HtmlPxMultiplier, HtmlOuterBorder, HtmlOuterPadding, HtmlOuterWidth, HtmlTextAlign, Background, TextColor, \
    TruncValue, TextString, TableRowKey, ColumnDivider, HtmlDividerWeight, BackgroundPadding


@dataclass
class OuterBase:
    outer_border: Optional[OuterBorder] = outer_border_default
    outer_padding: Optional[OuterPadding] = outer_padding_default
    outer_width: Optional[OuterWidth] = outer_width_default


###################################

@dataclass
class FrameBase:
    frame_divider: Optional[FrameDivider] = frame_divider_default
    max_lines: Optional[MaxLines] = max_lines_default
    multiline: Optional[Multiline] = multiline_default
    background: Optional[Background] = background_default
    trunc_value: Optional[TruncValue] = trunc_value_default


@dataclass
class ColumnBase:
    divider: Optional[ColumnDivider] = column_divider_default
    padding: Optional[ColumnPadding] = column_padding_default
    background_padding: Optional[BackgroundPadding] = background_default


@dataclass
class TextBase:
    text_style: Optional[TextStyle] = text_style_default
    text_align: Optional[TextAlign] = text_align_default
    text_color: Optional[TextColor] = text_color_default


###################################
###################################
###################################

TableHeaderFrameBase = FrameBase


@dataclass
class TableRowsBase:
    row_line_divider: Optional[FrameDivider] = row_line_divider_default
    odds_background: Optional[Background] = background_default
    evens_background: Optional[Background] = background_default


########################################################################################################################
########################################################################################################################
########################################################################################################################

@dataclass
class HtmlOuterBase:
    html_outer_border_weight: Optional[HtmlDividerWeight] = html_divider_weight_default
    html_outer_border_style: Optional[HtmlOuterBorder] = html_outer_border_default
    html_outer_padding: Optional[HtmlOuterPadding] = field(default_factory=lambda: [html_outer_padding_default])
    html_outer_width: Optional[HtmlOuterWidth] = html_outer_width_default

@dataclass
class HtmlFrameBase:
    html_frame_divider_style: Optional[HtmlFrameDivider] = html_frame_divider_default
    html_frame_divider_weight: Optional[HtmlDividerWeight] = html_divider_weight_default
    html_max_lines: Optional[MaxLines] = html_max_lines_default
    html_multiline: Optional[Multiline] = html_multiline_default
    html_background: Optional[HtmlBackground] = html_background_default


@dataclass
class HtmlColumnBase:
    html_divider_style: Optional[HtmlColumnDividerStyle] = column_divider_default
    html_divider_weight: Optional[HtmlDividerWeight] = html_divider_weight_default
    html_padding: Optional[HtmlColumnPadding] = column_padding_default


@dataclass
class HtmlTextBase:
    html_text_style: Optional[HtmlTextStyle] = html_text_style_default
    html_text_align: Optional[HtmlTextAlign] = html_text_align_default
    html_text_color: Optional[HtmlTextColor] = html_text_color_default
    html_text_size: Optional[HtmlTextSize] = html_text_size_default


###################################
###################################
###################################


@dataclass
class HtmlTableRowsBase:
    html_row_line_divider_weight: Optional[HtmlDividerWeight] = html_divider_weight_default
    html_row_line_divider_style: Optional[HtmlFrameDivider] = html_row_line_divider_style_default
    html_odds_background: Optional[HtmlBackground] = html_background_default
    html_evens_background: Optional[HtmlBackground] = html_background_default






from tablate.classes.options.html.style.subclasses.ElementStyler import ElementStyler
from tablate.library.checkers.is_last_element import is_last_element
from tablate.library.checkers.set_key_resolver import set_key_resolver
from tablate.library.formatters.html.style.attributes.color import background_color_attr
from tablate.library.formatters.html.style.attributes.divider import divider_attr
from tablate.library.formatters.html.style.attributes.padding import padding_attr
from tablate.library.formatters.html.style.elements.style_text import style_text
from tablate.type.defaults import html_padding_default
from tablate.type.primitives import HtmlPxMultiplier
from tablate.type.type_base import HtmlColumnBase, HtmlTextBase
from tablate.type.type_input import BaseColumnStyles
from tablate.type.type_store import BaseFrameStore, BaseColumnStore, FrameStoreUnion


def style_column(column_store: HtmlColumnBase,
                 column_styler: ElementStyler,
                 html_px_multiplier: HtmlPxMultiplier) -> None:
    column_padding = column_store.html_padding
    column_padding_css = ""
    column_divider_weight = column_store.html_divider_weight
    column_divider_style = column_store.html_divider_style

    padding_string = padding_attr(column_padding=column_padding,
                                  html_px_multiplier=html_px_multiplier)

    column_styler.add_style_attribute("padding", padding_string)
    if column_divider_style is not None or column_divider_weight is not None:
        column_styler.add_style_attribute(attribute="border-right",
                                          value=divider_attr(divider_style=column_divider_style,
                                                             divider_weight=column_divider_weight),
                                          sub_selector=":not(:last-child)")


def style_column_dict(column_dict: BaseColumnStyles, column_styler: ElementStyler, html_px_multiplier: HtmlPxMultiplier) -> None:
    if "html_styles" in column_dict and column_dict["html_styles"] is not None:
        column_padding = set_key_resolver(instance=column_dict["html_styles"], key="padding", default=html_padding_default)
        divider_style = set_key_resolver(instance=column_dict["html_styles"], key="divider_style", default=None)
        divider_weight = set_key_resolver(instance=column_dict["html_styles"], key="divider_weight", default=None)
        column_background = set_key_resolver(instance=column_dict["html_styles"], key="background", default=None)

        style_column(column_store=HtmlColumnBase(html_padding=column_padding,
                                                 html_divider_style=divider_style,
                                                 html_divider_weight=divider_weight),
                     column_styler=column_styler,
                     html_px_multiplier=html_px_multiplier)

        if column_background is not None:
            column_styler.add_style_attribute("background-color", background_color_attr(column_background))

        text_align = set_key_resolver(instance=column_dict["html_styles"], key="text_align", default=None)
        text_size = set_key_resolver(instance=column_dict["html_styles"], key="text_size", default=None)
        text_style = set_key_resolver(instance=column_dict["html_styles"], key="text_style", default=None)
        text_color = set_key_resolver(instance=column_dict["html_styles"], key="text_color", default=None)

        text_styler = column_styler.text
        style_text(text_store=HtmlTextBase(html_text_align=text_align,
                                           html_text_size=text_size,
                                           html_text_style=text_style,
                                           html_text_color=text_color),
                   text_styler=text_styler,
                   html_px_multiplier=html_px_multiplier)



from tablate.classes.options.html.style.CssStyler import CssStyler
from tablate.library.calcs.calc_column_percent import calc_column_percent
from tablate.library.initializers.append.append import process_frame_list
from tablate.library.initializers.globals_init import globals_init
from tablate.library.renderers.html.render_html_foot import render_html_foot
from tablate.library.renderers.html.render_html_frames import render_html_frames
from tablate.library.renderers.html.render_html_head import render_html_head
from tablate.type.type_store import FrameStoreList
from tablate.type.type_global import Globals


def render_html(frame_list: dict, globals_dict: dict, css_injection: list) -> str:
    global_options = globals_init(**globals_dict)
    processed_frame_list = process_frame_list(frame_list=frame_list, global_options=global_options)

    global_options.html.styler = CssStyler()

    for css_injection in css_injection:
        global_options.html.styler.inject_scoped_css(css_injection["selector"], css_injection["css"], css_injection["sub_selector"])

    global_options.html.styler.inject_css_block(global_options.html.css_injection)

    return_html = ""

    if len(processed_frame_list) > 0:
        processed_frame_list, column_baselines = calc_column_percent(frame_list=processed_frame_list,
                                                                     outer_width=global_options.console.outer_styles.outer_width)
        global_options.html.column_baselines = column_baselines
        return_html += render_html_head(global_options=global_options)
        return_html += render_html_frames(frame_list=processed_frame_list, global_options=global_options)
        return_html += render_html_foot()

    css_head = global_options.html.styler.return_head_styles()
    css_foot = global_options.html.styler.return_foot_styles()

    return_html = f"{css_head}{return_html}{css_foot}"

    return return_html
